#include <lwserver.h>
#include <lwhost.h>
#include <lwcmdseq.h>
#include <lwpanel.h>

#include <lwsurf.h>
#include <lwdyna.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <io.h>
#include <math.h>

#include "nanosvg.h"

#define MAXFILESZ 520

LWMessageFuncs *msgf;
int xdet = 10;

typedef struct st_surfaces
{
	int color;
	char name[250];
}structsurface;

static LWValue ival={LWT_INTEGER},ivecval={LWT_VINT},fval={LWT_FLOAT},fvecval={LWT_VFLOAT},sval={LWT_STRING};
XCALL_( int )
Activate( long version, GlobalFunc *global, LWModCommand *local,
   void *serverData )
{
	LWPanelFuncs *panf;
	LWSurfaceFuncs *surff;
	LWFileActivateFunc *filereq;
    LWStateQueryFuncs *query;



	MeshEditOp *edit;



	LWPanelID panel;
	LWControl *ctl;
	LWPanControlDesc desc;

	char cm[256],dm[256];
	char filename[512];
	int numofshapes;
	int numofpath;
	int numofpoints;
	int surfacecount;
	int surfinuse;
	int detout;
	structsurface *surface;
	float detail;  
	static LWFileReqLocal frloc;
	static char
		fnode[ MAXFILESZ ] = "",
		fpath[ MAXFILESZ ] = "",
		fname[ MAXFILESZ ] = "";
	int t,x,i,y;
	int result;
	LWPntID *pntid;
	

   if ( version != LWMODCOMMAND_VERSION )
      return AFUNC_BADVERSION;

	panf = (LWPanelFuncs *)global( LWPANELFUNCS_GLOBAL, GFUSE_TRANSIENT );
	surff = (LWSurfaceFuncs *)global( LWSURFACEFUNCS_GLOBAL, GFUSE_TRANSIENT );
	msgf = (LWMessageFuncs *)global( LWMESSAGEFUNCS_GLOBAL, GFUSE_TRANSIENT );
	filereq = (LWFileActivateFunc *)global( LWFILEACTIVATEFUNC_GLOBAL, GFUSE_TRANSIENT );
    query = (LWStateQueryFuncs *)global( LWSTATEQUERYFUNCS_GLOBAL, GFUSE_TRANSIENT );
	frloc.reqType  = FREQ_LOAD; 
	frloc.title    = "Load svg";
	frloc.bufLen   = MAXFILESZ;
	frloc.pickName = NULL;
	frloc.fileType = "";
	frloc.path     = fpath;
	frloc.baseName = fnode;
	frloc.fullName = fname;		
	result = filereq( LWFILEREQ_VERSION, &frloc );

	if ( result != AFUNC_OK && frloc.result < 0 ) {   return AFUNC_OK;}

	sprintf(filename,"%s",frloc.fullName);

	//Add Panel here

	panel = PAN_CREATE( panf, "SVG Import (c)2014 chris huf" );
	if ( !panel ) return AFUNC_BADGLOBAL;
	ctl = SLIDER_CTL( panf, panel,"Bezier Detail",100, 1, 1000 );
	SET_INT( ctl, xdet );

	if ( panf->open( panel, PANF_BLOCKING | PANF_CANCEL ))
	{
      GET_INT( ctl, detout);
		xdet = detout;
	}
	else
	{
		return AFUNC_OK;
	}

	if(xdet ==0)
	{detail = 1.0;}else{detail = 1.0/(float)xdet;}


	PAN_KILL( panf, panel );

	struct SVGPath* plist;
	
	plist = svgParseFromFile(filename,detail);
	edit = local->editBegin( 0, 0, OPSEL_GLOBAL );



	//allocate a struct to contain all surfaces needed

	surfacecount =0;
	for (SVGPath* it = plist; it; it = it->next)
	{
		surfacecount++;
	}
	surface = (structsurface*)malloc(surfacecount*sizeof(structsurface));

	surfinuse =0;
	y=0;

	//generate the paths
	//check if the path color is actually used before
	//and apply the surface name
	for (SVGPath* it = plist; it; it = it->next)
	{
		LWPntID *pntid;
		pntid = (LWPntID *)malloc(it->npts*sizeof(LWPntID));

		x= -1;
		for(t=0; t< surfinuse; t++)
		{
			if(it->fillColor == surface[t].color)
			{
				x = t;		
			}
		}
		if(x == -1)
		{
			surface[surfinuse].color = it->fillColor ;
			x= surfinuse;
			sprintf(surface[surfinuse].name,"Surface %i",x);
			surfinuse++;
		}
			sprintf(cm,"Surface %i",x);

		for (int i = 0; i < it->npts; i++)
		{
			
			LWDVector xpos;
			xpos[0] = it->pts[i*2];
			xpos[1] = -it->pts[i*2+1];
			xpos[2] = 0.0;
			pntid[i] = edit->addPoint(edit->state,xpos);			
		}


		edit->addPoly(edit->state,NULL,NULL,cm,it->npts,pntid);	

		free(pntid);
	}
	edit->done( edit->state, EDERR_NONE, 0 );


	//edit the surface color _after_ the mesh edit operation is done

	for (SVGPath* it = plist; it; it = it->next)
	{
		for(t=0; t< surfinuse;t++)
		{
			if(surface[t].color == it->fillColor)
			{
				int red, green, blue, alpha;
				double color[3];
				double trans;
				LWSurfaceID srfid;

				red = it->fillColor & 0xff;
				green = (it->fillColor >> 8) & 0xff;
				blue = (it->fillColor >> 16) & 0xff;
				alpha = (it->fillColor >> 24) & 0xff;


				//for some reason red an blue is switched
				color[2] = (double)red / 255.0;
				color[1] = (double)green / 255.0;
				color[0] = (double)blue / 255.0;
				trans = 1.0 - ((double)alpha / 255.0);

				srfid = surff->first();
				while(srfid)
				{
					if(strcmp((const char*)surff->name(srfid),surface[t].name)==0)
					{
						surff->setFlt(srfid,SURF_TRAN,&trans);	
						surff->setFlt(srfid,SURF_COLR,color);	
					}
					srfid = surff->next(srfid);
				}

			}
		}
	}






	// Delete
	svgDelete(plist);
	free(surface);
   return AFUNC_OK;
}



void *Startup( void )
{
	// Zero return is error
    return( ( void * )4 );
}


void Shutdown( void *serverData )
{
}
extern "C" {
ServerRecord ServerDesc[] = {
   { LWMODCOMMAND_CLASS, "tibe3_svgimport",(ActivateFunc (__cdecl *)) Activate },
   { NULL }
};
}